@@ -44,7 +44,7 @@
    #include <botan/system_rng.h>
 #endif
 
-#if defined(BOTAN_HAS_AUTO_SEEDED_RNG)
+#if defined(BOTAN_HAS_AUTO_SEEDING_RNG) && defined(BOTAN_AUTO_RNG_HMAC)
    #include <botan/auto_rng.h>
 #endif
 
@@ -96,8 +96,8 @@ class Timing_Test
          {
 #if defined(BOTAN_HAS_SYSTEM_RNG)
          return Botan::system_rng();
-#elif defined(BOTAN_HAS_AUTO_SEEDED_RNG)
-         static AutoSeeded_RNG static_timing_test_rng(Botan::Entropy_Sources::global_sources(), 0);
+#elif defined(BOTAN_HAS_AUTO_SEEDING_RNG) && defined(BOTAN_AUTO_RNG_HMAC)
+         static Botan::AutoSeeded_RNG static_timing_test_rng;
          return static_timing_test_rng;
 #else
          // we could just use SHA-256 in OFB mode for these purposes
@@ -107,7 +107,7 @@ class Timing_Test
 
    };
 
-#if defined(BOTAN_HAS_RSA) && defined(BOTAN_HAS_EME_PKCS1v15) && defined(BOTAN_HAS_EME_RAW)
+#if defined(BOTAN_HAS_RSA) && defined(BOTAN_HAS_EME_PKCS1) && defined(BOTAN_HAS_EME_RAW)
 
 class Bleichenbacker_Timing_Test final : public Timing_Test
    {
@@ -209,7 +209,7 @@ class Lucky13_Timing_Test final : public Timing_Test
          , m_mac_keylen(mac_keylen)
          , m_dec(Botan::BlockCipher::create_or_throw("AES-128"),
                  Botan::MessageAuthenticationCode::create_or_throw("HMAC(" + m_mac_algo + ")"),
-                 16, m_mac_keylen, true, false) {}
+                 16, m_mac_keylen, Botan::TLS::Protocol_Version::TLS_V11, false) {}
 
       std::vector<uint8_t> prepare_input(std::string input) override;
       ticks measure_critical_function(std::vector<uint8_t> input) override;
@@ -544,7 +544,7 @@ BOTAN_REGISTER_COMMAND("timing_test", Timing_Test_Command);
 
 std::unique_ptr<Timing_Test> Timing_Test_Command::lookup_timing_test(const std::string& test_type)
    {
-#if defined(BOTAN_HAS_RSA) && defined(BOTAN_HAS_EME_PKCS1v15) && defined(BOTAN_HAS_EME_RAW)
+#if defined(BOTAN_HAS_RSA) && defined(BOTAN_HAS_EME_PKCS1) && defined(BOTAN_HAS_EME_RAW)
    if(test_type == "bleichenbacher")
       {
       return std::unique_ptr<Timing_Test>(new Bleichenbacker_Timing_Test(2048));
