@@ -2,7 +2,7 @@
 * Point arithmetic on elliptic curves over GF(p)
 *
 * (C) 2007 Martin Doering, Christoph Ludwig, Falko Strenzke
-*     2008-2011,2012,2014,2015 Jack Lloyd
+*     2008-2011,2012,2014,2015,2018 Jack Lloyd
 *
 * Botan is released under the Simplified BSD License (see license.txt)
 */
@@ -11,26 +11,24 @@
 #include <botan/numthry.h>
 #include <botan/rng.h>
 #include <botan/internal/rounding.h>
+#include <botan/internal/ct_utils.h>
 
 namespace Botan {
 
 PointGFp::PointGFp(const CurveGFp& curve) :
    m_curve(curve),
    m_coord_x(0),
-   m_coord_y(1),
+   m_coord_y(curve.get_1_rep()),
    m_coord_z(0)
    {
-   secure_vector<word> monty_ws(m_curve.get_ws_size());
-   m_curve.to_rep(m_coord_x, monty_ws);
-   m_curve.to_rep(m_coord_y, monty_ws);
-   m_curve.to_rep(m_coord_z, monty_ws);
+   // Assumes Montgomery rep of zero is zero
    }
 
 PointGFp::PointGFp(const CurveGFp& curve, const BigInt& x, const BigInt& y) :
    m_curve(curve),
    m_coord_x(x),
    m_coord_y(y),
-   m_coord_z(1)
+   m_coord_z(m_curve.get_1_rep())
    {
    if(x <= 0 || x >= curve.get_p())
       throw Invalid_Argument("Invalid PointGFp affine x");
@@ -40,7 +38,6 @@ PointGFp::PointGFp(const CurveGFp& curve, const BigInt& x, const BigInt& y) :
    secure_vector<word> monty_ws(m_curve.get_ws_size());
    m_curve.to_rep(m_coord_x, monty_ws);
    m_curve.to_rep(m_coord_y, monty_ws);
-   m_curve.to_rep(m_coord_z, monty_ws);
    }
 
 void PointGFp::randomize_repr(RandomNumberGenerator& rng)
@@ -51,20 +48,21 @@ void PointGFp::randomize_repr(RandomNumberGenerator& rng)
 
 void PointGFp::randomize_repr(RandomNumberGenerator& rng, secure_vector<word>& ws)
    {
-   if(BOTAN_POINTGFP_RANDOMIZE_BLINDING_BITS > 1)
-      {
-      BigInt mask;
-      while(mask.is_zero())
-         mask.randomize(rng, BOTAN_POINTGFP_RANDOMIZE_BLINDING_BITS, false);
+   const BigInt mask = BigInt::random_integer(rng, 2, m_curve.get_p());
 
-      //m_curve.to_rep(mask, ws);
-      const BigInt mask2 = m_curve.sqr_to_tmp(mask, ws);
-      const BigInt mask3 = m_curve.mul_to_tmp(mask2, mask, ws);
+   /*
+   * No reason to convert this to Montgomery representation first,
+   * just pretend the random mask was chosen as Redc(mask) and the
+   * random mask we generated above is in the Montgomery
+   * representation.
+   * //m_curve.to_rep(mask, ws);
+   */
+   const BigInt mask2 = m_curve.sqr_to_tmp(mask, ws);
+   const BigInt mask3 = m_curve.mul_to_tmp(mask2, mask, ws);
 
-      m_coord_x = m_curve.mul_to_tmp(m_coord_x, mask2, ws);
-      m_coord_y = m_curve.mul_to_tmp(m_coord_y, mask3, ws);
-      m_coord_z = m_curve.mul_to_tmp(m_coord_z, mask, ws);
-      }
+   m_coord_x = m_curve.mul_to_tmp(m_coord_x, mask2, ws);
+   m_coord_y = m_curve.mul_to_tmp(m_coord_y, mask3, ws);
+   m_coord_z = m_curve.mul_to_tmp(m_coord_z, mask, ws);
    }
 
 namespace {
@@ -79,52 +77,43 @@ inline void resize_ws(std::vector<BigInt>& ws_bn, size_t cap_size)
          ws_bn[i].get_word_vector().resize(cap_size);
    }
 
-inline bool all_zeros(const word x[], size_t len)
+inline word all_zeros(const word x[], size_t len)
    {
    word z = 0;
    for(size_t i = 0; i != len; ++i)
       z |= x[i];
-   return (z == 0);
+   return CT::Mask<word>::is_zero(z).value();
    }
 
 }
 
-void PointGFp::add_affine(const PointGFp& rhs, std::vector<BigInt>& workspace)
-   {
-   BOTAN_DEBUG_ASSERT(rhs.is_affine());
-
-   const size_t p_words = m_curve.get_p_words();
-   add_affine(rhs.m_coord_x.data(), std::min(p_words, rhs.m_coord_x.size()),
-              rhs.m_coord_y.data(), std::min(p_words, rhs.m_coord_y.size()),
-              workspace);
-   }
-
 void PointGFp::add_affine(const word x_words[], size_t x_size,
                           const word y_words[], size_t y_size,
                           std::vector<BigInt>& ws_bn)
    {
-   if(all_zeros(x_words, x_size) && all_zeros(y_words, y_size))
+   if(all_zeros(x_words, x_size) & all_zeros(y_words, y_size))
+      {
       return;
+      }
 
    if(is_zero())
       {
-      // FIXME avoid the copy here
-      m_coord_x = BigInt(x_words, x_size);
-      m_coord_y = BigInt(y_words, y_size);
-      m_coord_z = 1;
-      m_curve.to_rep(m_coord_z, ws_bn[0].get_word_vector());
+      m_coord_x.set_words(x_words, x_size);
+      m_coord_y.set_words(y_words, y_size);
+      m_coord_z = m_curve.get_1_rep();
       return;
       }
 
    resize_ws(ws_bn, m_curve.get_ws_size());
 
    secure_vector<word>& ws = ws_bn[0].get_word_vector();
+   secure_vector<word>& sub_ws = ws_bn[1].get_word_vector();
 
-   BigInt& T0 = ws_bn[1];
-   BigInt& T1 = ws_bn[2];
-   BigInt& T2 = ws_bn[3];
-   BigInt& T3 = ws_bn[4];
-   BigInt& T4 = ws_bn[5];
+   BigInt& T0 = ws_bn[2];
+   BigInt& T1 = ws_bn[3];
+   BigInt& T2 = ws_bn[4];
+   BigInt& T3 = ws_bn[5];
+   BigInt& T4 = ws_bn[6];
 
    /*
    https://hyperelliptic.org/EFD/g1p/auto-shortw-jacobian-3.html#addition-add-1998-cmo-2
@@ -139,13 +128,9 @@ void PointGFp::add_affine(const word x_words[], size_t x_size,
    m_curve.mul(T2, m_coord_z, T3, ws); // z1^3
    m_curve.mul(T0, y_words, y_size, T2, ws); // y2*z1^3
 
-   T4 -= m_coord_x; // x2*z1^2 - x1*z2^2
-   if(T4.is_negative())
-      T4 += p;
+   T4.mod_sub(m_coord_x, p, sub_ws); // x2*z1^2 - x1*z2^2
 
-   T0 -= m_coord_y;
-   if(T0.is_negative())
-      T0 += p;
+   T0.mod_sub(m_coord_y, p, sub_ws);
 
    if(T4.is_zero())
       {
@@ -156,9 +141,9 @@ void PointGFp::add_affine(const word x_words[], size_t x_size,
          }
 
       // setting to zero:
-      m_coord_x = 0;
-      m_coord_y = 1;
-      m_coord_z = 0;
+      m_coord_x.clear();
+      m_coord_y = m_curve.get_1_rep();
+      m_coord_z.clear();
       return;
       }
 
@@ -169,52 +154,49 @@ void PointGFp::add_affine(const word x_words[], size_t x_size,
    m_curve.mul(T1, T2, T4, ws);
 
    m_curve.sqr(m_coord_x, T0, ws);
-   m_coord_x -= T1;
-   m_coord_x -= T3;
-   m_coord_x -= T3;
-   while(m_coord_x.is_negative())
-      m_coord_x += p;
+   m_coord_x.mod_sub(T1, p, sub_ws);
 
-   T3 -= m_coord_x;
-   if(T3.is_negative())
-      T3 += p;
+   m_coord_x.mod_sub(T3, p, sub_ws);
+   m_coord_x.mod_sub(T3, p, sub_ws);
+
+   T3.mod_sub(m_coord_x, p, sub_ws);
 
-   T2 = m_coord_y;
    m_curve.mul(T2, T0, T3, ws);
-   m_curve.mul(T3, m_coord_y, T1, ws);
-   T2 -= T3;
-   if(T2.is_negative())
-      T2 += p;
-   m_coord_y = T2;
+   m_curve.mul(T0, m_coord_y, T1, ws);
+   T2.mod_sub(T0, p, sub_ws);
+   m_coord_y.swap(T2);
 
-   m_curve.mul(T3, m_coord_z, T4, ws);
-   m_coord_z = T3;
+   m_curve.mul(T0, m_coord_z, T4, ws);
+   m_coord_z.swap(T0);
    }
 
-// Point addition
-void PointGFp::add(const PointGFp& rhs, std::vector<BigInt>& ws_bn)
+void PointGFp::add(const word x_words[], size_t x_size,
+                   const word y_words[], size_t y_size,
+                   const word z_words[], size_t z_size,
+                   std::vector<BigInt>& ws_bn)
    {
-   if(rhs.is_zero())
+   if(all_zeros(x_words, x_size) & all_zeros(z_words, z_size))
       return;
 
    if(is_zero())
       {
-      m_coord_x = rhs.m_coord_x;
-      m_coord_y = rhs.m_coord_y;
-      m_coord_z = rhs.m_coord_z;
+      m_coord_x.set_words(x_words, x_size);
+      m_coord_y.set_words(y_words, y_size);
+      m_coord_z.set_words(z_words, z_size);
       return;
       }
 
    resize_ws(ws_bn, m_curve.get_ws_size());
 
    secure_vector<word>& ws = ws_bn[0].get_word_vector();
+   secure_vector<word>& sub_ws = ws_bn[1].get_word_vector();
 
-   BigInt& T0 = ws_bn[1];
-   BigInt& T1 = ws_bn[2];
-   BigInt& T2 = ws_bn[3];
-   BigInt& T3 = ws_bn[4];
-   BigInt& T4 = ws_bn[5];
-   BigInt& T5 = ws_bn[6];
+   BigInt& T0 = ws_bn[2];
+   BigInt& T1 = ws_bn[3];
+   BigInt& T2 = ws_bn[4];
+   BigInt& T3 = ws_bn[5];
+   BigInt& T4 = ws_bn[6];
+   BigInt& T5 = ws_bn[7];
 
    /*
    https://hyperelliptic.org/EFD/g1p/auto-shortw-jacobian-3.html#addition-add-1998-cmo-2
@@ -222,66 +204,75 @@ void PointGFp::add(const PointGFp& rhs, std::vector<BigInt>& ws_bn)
 
    const BigInt& p = m_curve.get_p();
 
-   m_curve.sqr(T0, rhs.m_coord_z, ws); // z2^2
+   m_curve.sqr(T0, z_words, z_size, ws); // z2^2
    m_curve.mul(T1, m_coord_x, T0, ws); // x1*z2^2
-   m_curve.mul(T3, rhs.m_coord_z, T0, ws); // z2^3
+   m_curve.mul(T3, z_words, z_size, T0, ws); // z2^3
    m_curve.mul(T2, m_coord_y, T3, ws); // y1*z2^3
 
    m_curve.sqr(T3, m_coord_z, ws); // z1^2
-   m_curve.mul(T4, rhs.m_coord_x, T3, ws); // x2*z1^2
+   m_curve.mul(T4, x_words, x_size, T3, ws); // x2*z1^2
 
    m_curve.mul(T5, m_coord_z, T3, ws); // z1^3
-   m_curve.mul(T0, rhs.m_coord_y, T5, ws); // y2*z1^3
+   m_curve.mul(T0, y_words, y_size, T5, ws); // y2*z1^3
 
-   T4 -= T1; // x2*z1^2 - x1*z2^2
-   if(T4.is_negative())
-      T4 += p;
+   T4.mod_sub(T1, p, sub_ws); // x2*z1^2 - x1*z2^2
 
-   T3 = T0;
-   T3 -= T2;
-   if(T3.is_negative())
-      T3 += p;
+   T0.mod_sub(T2, p, sub_ws);
 
    if(T4.is_zero())
       {
-      if(T3.is_zero())
+      if(T0.is_zero())
          {
          mult2(ws_bn);
          return;
          }
 
       // setting to zero:
-      m_coord_x = 0;
-      m_coord_y = 1;
-      m_coord_z = 0;
+      m_coord_x.clear();
+      m_coord_y = m_curve.get_1_rep();
+      m_coord_z.clear();
       return;
       }
 
    m_curve.sqr(T5, T4, ws);
 
-   m_curve.mul(T0, T1, T5, ws);
+   m_curve.mul(T3, T1, T5, ws);
 
    m_curve.mul(T1, T5, T4, ws);
 
-   m_curve.sqr(m_coord_x, T3, ws);
-   m_coord_x -= T1;
-   m_coord_x -= T0;
-   m_coord_x -= T0;
-   while(m_coord_x.is_negative())
-      m_coord_x += p;
+   m_curve.sqr(m_coord_x, T0, ws);
+   m_coord_x.mod_sub(T1, p, sub_ws);
+   m_coord_x.mod_sub(T3, p, sub_ws);
+   m_coord_x.mod_sub(T3, p, sub_ws);
 
-   T0 -= m_coord_x;
-   if(T0.is_negative())
-      T0 += p;
+   T3.mod_sub(m_coord_x, p, sub_ws);
 
-   m_curve.mul(m_coord_y, T3, T0, ws);
-   m_curve.mul(T0, T2, T1, ws);
-   m_coord_y -= T0;
-   if(m_coord_y.is_negative())
-      m_coord_y += p;
+   m_curve.mul(m_coord_y, T0, T3, ws);
+   m_curve.mul(T3, T2, T1, ws);
 
-   m_curve.mul(T0, m_coord_z, rhs.m_coord_z, ws);
-   m_curve.mul(m_coord_z, T0, T4, ws);
+   m_coord_y.mod_sub(T3, p, sub_ws);
+
+   m_curve.mul(T3, z_words, z_size, m_coord_z, ws);
+   m_curve.mul(m_coord_z, T3, T4, ws);
+   }
+
+void PointGFp::mult2i(size_t iterations, std::vector<BigInt>& ws_bn)
+   {
+   if(iterations == 0)
+      return;
+
+   if(m_coord_y.is_zero())
+      {
+      *this = PointGFp(m_curve); // setting myself to zero
+      return;
+      }
+
+   /*
+   TODO we can save 2 squarings per iteration by computing
+   a*Z^4 using values cached from previous iteration
+   */
+   for(size_t i = 0; i != iterations; ++i)
+      mult2(ws_bn);
    }
 
 // *this *= 2
@@ -299,11 +290,13 @@ void PointGFp::mult2(std::vector<BigInt>& ws_bn)
    resize_ws(ws_bn, m_curve.get_ws_size());
 
    secure_vector<word>& ws = ws_bn[0].get_word_vector();
-   BigInt& T0 = ws_bn[1];
-   BigInt& T1 = ws_bn[2];
-   BigInt& T2 = ws_bn[6];
-   BigInt& T3 = ws_bn[4];
-   BigInt& T4 = ws_bn[5];
+   secure_vector<word>& sub_ws = ws_bn[1].get_word_vector();
+
+   BigInt& T0 = ws_bn[2];
+   BigInt& T1 = ws_bn[3];
+   BigInt& T2 = ws_bn[4];
+   BigInt& T3 = ws_bn[5];
+   BigInt& T4 = ws_bn[6];
 
    /*
    https://hyperelliptic.org/EFD/g1p/auto-shortw-jacobian-3.html#doubling-dbl-1986-cc
@@ -313,45 +306,63 @@ void PointGFp::mult2(std::vector<BigInt>& ws_bn)
    m_curve.sqr(T0, m_coord_y, ws);
 
    m_curve.mul(T1, m_coord_x, T0, ws);
-   T1 <<= 2; // * 4
-   T1.reduce_below(p, T3.get_word_vector());
+   T1.mod_mul(4, p, sub_ws);
 
-   m_curve.sqr(T3, m_coord_z, ws); // z^2
-   m_curve.sqr(T4, T3, ws); // z^4
-   m_curve.mul(T3, m_curve.get_a_rep(), T4, ws);
+   if(m_curve.a_is_zero())
+      {
+      // if a == 0 then 3*x^2 + a*z^4 is just 3*x^2
+      m_curve.sqr(T4, m_coord_x, ws); // x^2
+      T4.mod_mul(3, p, sub_ws); // 3*x^2
+      }
+   else if(m_curve.a_is_minus_3())
+      {
+      /*
+      if a == -3 then
+        3*x^2 + a*z^4 == 3*x^2 - 3*z^4 == 3*(x^2-z^4) == 3*(x-z^2)*(x+z^2)
+      */
+      m_curve.sqr(T3, m_coord_z, ws); // z^2
 
-   m_curve.sqr(T4, m_coord_x, ws);
-   T4 *= 3;
-   T4 += T3;
-   T4.reduce_below(p, T3.get_word_vector());
+      // (x-z^2)
+      T2 = m_coord_x;
+      T2.mod_sub(T3, p, sub_ws);
+
+      // (x+z^2)
+      T3.mod_add(m_coord_x, p, sub_ws);
+
+      m_curve.mul(T4, T2, T3, ws); // (x-z^2)*(x+z^2)
+
+      T4.mod_mul(3, p, sub_ws); // 3*(x-z^2)*(x+z^2)
+      }
+   else
+      {
+      m_curve.sqr(T3, m_coord_z, ws); // z^2
+      m_curve.sqr(T4, T3, ws); // z^4
+      m_curve.mul(T3, m_curve.get_a_rep(), T4, ws); // a*z^4
+
+      m_curve.sqr(T4, m_coord_x, ws); // x^2
+      T4.mod_mul(3, p, sub_ws);
+      T4.mod_add(T3, p, sub_ws); // 3*x^2 + a*z^4
+      }
 
    m_curve.sqr(T2, T4, ws);
-   T2 -= T1;
-   T2 -= T1;
-   while(T2.is_negative())
-      T2 += p;
+   T2.mod_sub(T1, p, sub_ws);
+   T2.mod_sub(T1, p, sub_ws);
 
    m_curve.sqr(T3, T0, ws);
-   T3 <<= 3;
-   T3.reduce_below(p, T0.get_word_vector());
+   T3.mod_mul(8, p, sub_ws);
 
-   T1 -= T2;
-   while(T1.is_negative())
-      T1 += p;
+   T1.mod_sub(T2, p, sub_ws);
 
    m_curve.mul(T0, T4, T1, ws);
-   T0 -= T3;
-   if(T0.is_negative())
-      T0 += p;
+   T0.mod_sub(T3, p, sub_ws);
 
-   m_coord_x = T2;
+   m_coord_x.swap(T2);
 
    m_curve.mul(T2, m_coord_y, m_coord_z, ws);
-   T2 <<= 1;
-   T2.reduce_below(p, T3.get_word_vector());
+   T2.mod_mul(2, p, sub_ws);
 
-   m_coord_y = T0;
-   m_coord_z = T2;
+   m_coord_y.swap(T0);
+   m_coord_z.swap(T2);
    }
 
 // arithmetic operators
@@ -426,13 +437,11 @@ void PointGFp::force_all_affine(std::vector<PointGFp>& points,
    */
 
    const CurveGFp& curve = points[0].m_curve;
+   const BigInt& rep_1 = curve.get_1_rep();
 
    if(ws.size() < curve.get_ws_size())
       ws.resize(curve.get_ws_size());
 
-   BigInt rep_1 = 1;
-   curve.to_rep(rep_1, ws);
-
    std::vector<BigInt> c(points.size());
    c[0] = points[0].m_coord_z;
 
@@ -479,8 +488,7 @@ void PointGFp::force_affine()
    const BigInt z3_inv = m_curve.mul_to_tmp(z_inv, z2_inv, ws);
    m_coord_x = m_curve.mul_to_tmp(m_coord_x, z2_inv, ws);
    m_coord_y = m_curve.mul_to_tmp(m_coord_y, z3_inv, ws);
-   m_coord_z = 1;
-   m_curve.to_rep(m_coord_z, ws);
+   m_coord_z = m_curve.get_1_rep();
    }
 
 bool PointGFp::is_affine() const
