@@ -11,16 +11,14 @@
 #if defined(BOTAN_TARGET_CPU_IS_PPC_FAMILY)
 
 /*
-* On Darwin and OpenBSD ppc, use sysctl to detect AltiVec
+* On macOS and OpenBSD ppc, use sysctl to detect AltiVec
 */
-#if defined(BOTAN_TARGET_OS_IS_DARWIN)
+#if defined(BOTAN_TARGET_OS_IS_MACOS)
   #include <sys/sysctl.h>
 #elif defined(BOTAN_TARGET_OS_IS_OPENBSD)
   #include <sys/param.h>
   #include <sys/sysctl.h>
   #include <machine/cpu.h>
-#elif defined(BOTAN_TARGET_OS_HAS_GETAUXVAL)
-  #include <sys/auxv.h>
 #endif
 
 #endif
@@ -33,12 +31,12 @@ namespace Botan {
 * PowerPC specific block: check for AltiVec using either
 * sysctl or by reading processor version number register.
 */
-uint64_t CPUID::detect_cpu_features(size_t* cache_line_size)
+uint64_t CPUID::CPUID_Data::detect_cpu_features(size_t* cache_line_size)
    {
    BOTAN_UNUSED(cache_line_size);
 
-#if defined(BOTAN_TARGET_OS_IS_DARWIN) || defined(BOTAN_TARGET_OS_IS_OPENBSD)
-   // On Darwin/OS X and OpenBSD, use sysctl
+#if defined(BOTAN_TARGET_OS_IS_MACOS) || defined(BOTAN_TARGET_OS_IS_OPENBSD)
+   // On macOS and OpenBSD, use sysctl
 
    int sels[2] = {
 #if defined(BOTAN_TARGET_OS_IS_OPENBSD)
@@ -55,7 +53,7 @@ uint64_t CPUID::detect_cpu_features(size_t* cache_line_size)
    if(error == 0 && vector_type > 0)
       return CPUID::CPUID_ALTIVEC_BIT;
 
-#elif defined(BOTAN_TARGET_OS_HAS_GETAUXVAL) && defined(BOTAN_TARGET_ARCH_IS_PPC64)
+#elif (defined(BOTAN_TARGET_OS_HAS_GETAUXVAL) || defined(BOTAN_TARGET_HAS_ELF_AUX_INFO)) && defined(BOTAN_TARGET_ARCH_IS_PPC64)
 
    enum PPC_hwcap_bit {
       ALTIVEC_bit  = (1 << 28),
@@ -67,11 +65,11 @@ uint64_t CPUID::detect_cpu_features(size_t* cache_line_size)
 
    uint64_t detected_features = 0;
 
-   const unsigned long hwcap_altivec = ::getauxval(PPC_hwcap_bit::ARCH_hwcap_altivec);
+   const unsigned long hwcap_altivec = OS::get_auxval(PPC_hwcap_bit::ARCH_hwcap_altivec);
    if(hwcap_altivec & PPC_hwcap_bit::ALTIVEC_bit)
       detected_features |= CPUID::CPUID_ALTIVEC_BIT;
 
-   const unsigned long hwcap_crypto = ::getauxval(PPC_hwcap_bit::ARCH_hwcap_crypto);
+   const unsigned long hwcap_crypto = OS::get_auxval(PPC_hwcap_bit::ARCH_hwcap_crypto);
    if(hwcap_crypto & PPC_hwcap_bit::CRYPTO_bit)
      detected_features |= CPUID::CPUID_PPC_CRYPTO_BIT;
 
