@@ -11,9 +11,14 @@
 #include <botan/exceptn.h>
 #include <botan/mem_ops.h>
 
+#include <algorithm>
 #include <chrono>
 #include <cstdlib>
 
+#if defined(BOTAN_TARGET_OS_HAS_THREADS)
+  #include <thread>
+#endif
+
 #if defined(BOTAN_TARGET_OS_HAS_EXPLICIT_BZERO)
   #include <string.h>
 #endif
@@ -35,7 +40,8 @@
   #include <emscripten/emscripten.h>
 #endif
 
-#if defined(BOTAN_TARGET_OS_HAS_GETAUXVAL)
+#if defined(BOTAN_TARGET_OS_HAS_GETAUXVAL) || defined(BOTAN_TARGET_OS_IS_ANDROID) || \
+  defined(BOTAN_TARGET_OS_HAS_ELF_AUX_INFO)
   #include <sys/auxv.h>
 #endif
 
@@ -44,6 +50,11 @@
   #include <windows.h>
 #endif
 
+#if defined(BOTAN_TARGET_OS_IS_ANDROID)
+  #include <elf.h>
+  extern "C" char **environ;
+#endif
+
 namespace Botan {
 
 // Not defined in OS namespace for historical reasons
@@ -90,10 +101,41 @@ uint32_t OS::get_process_id()
 #endif
    }
 
+unsigned long OS::get_auxval(unsigned long id)
+   {
+#if defined(BOTAN_TARGET_OS_HAS_GETAUXVAL)
+   return ::getauxval(id);
+#elif defined(BOTAN_TARGET_OS_IS_ANDROID)
+   char  **p = environ;
+
+   while (*p++!=nullptr);
+
+   Elf32_auxv_t *e = reinterpret_cast<Elf32_auxv_t *>(p);
+
+   while (e!=nullptr)
+      {
+      if (e->a_type == 0)
+        continue;
+      if (e->a_type == id)
+        return e->a_un.a_val;
+      e++;
+      }
+
+   return 0;
+#elif defined(BOTAN_TARGET_OS_HAS_ELF_AUX_INFO)
+   unsigned long auxinfo = 0;
+   ::elf_aux_info(id, &auxinfo, sizeof(auxinfo));
+   return auxinfo;
+#else
+   BOTAN_UNUSED(id);
+   return 0;
+#endif
+   }
+
 bool OS::running_in_privileged_state()
    {
-#if defined(BOTAN_TARGET_OS_HAS_GETAUXVAL) && defined(AT_SECURE)
-   return ::getauxval(AT_SECURE) != 0;
+#if defined(AT_SECURE)
+   return OS::get_auxval(AT_SECURE) != 0;
 #elif defined(BOTAN_TARGET_OS_HAS_POSIX1)
    return (::getuid() != ::geteuid()) || (::getgid() != ::getegid());
 #else
@@ -131,10 +173,10 @@ uint64_t OS::get_cpu_cycle_counter()
       asm volatile("mftbu %0" : "=r" (rtc_high2));
 
       if(rtc_high == rtc_high2)
-	 {
+         {
          rtc = (static_cast<uint64_t>(rtc_high) << 32) | rtc_low;
          break;
-	 }
+         }
       }
 
 #elif defined(BOTAN_TARGET_ARCH_IS_ALPHA)
@@ -162,6 +204,36 @@ uint64_t OS::get_cpu_cycle_counter()
    return rtc;
    }
 
+size_t OS::get_cpu_total()
+   {
+   size_t tt = 1;
+#if defined(BOTAN_TARGET_OS_HAS_POSIX1) && defined(_SC_NPROCESSORS_CONF)
+   long res;
+   if ((res = ::sysconf(_SC_NPROCESSORS_CONF)) <= 0)
+     {
+          return 1;
+     }
+     tt = static_cast<size_t>(res);
+#elif defined(BOTAN_TARGET_OS_HAS_THREADS)
+     tt = static_cast<size_t>(std::thread::hardware_concurrency());
+#endif
+     return tt;
+   }
+
+size_t OS::get_cpu_available()
+   {
+#if defined(BOTAN_TARGET_OS_HAS_POSIX1) && defined(_SC_NPROCESSORS_ONLN)
+   long res;
+   if ((res = ::sysconf(_SC_NPROCESSORS_ONLN)) <= 0)
+     {
+       return 1;
+     }
+   return static_cast<size_t>(res);
+#else
+   return get_cpu_total();
+#endif
+   }
+
 uint64_t OS::get_high_resolution_clock()
    {
    if(uint64_t cpu_clock = OS::get_cpu_cycle_counter())
@@ -262,20 +334,9 @@ size_t OS::get_memory_locking_limit()
    * programs), but small enough that we should not cause problems
    * even if many processes are mlocking on the same machine.
    */
-   size_t mlock_requested = BOTAN_MLOCK_ALLOCATOR_MAX_LOCKED_KB;
+   const size_t user_req = read_env_variable_sz("BOTAN_MLOCK_POOL_SIZE", BOTAN_MLOCK_ALLOCATOR_MAX_LOCKED_KB);
 
-   /*
-   * Allow override via env variable
-   */
-   if(const char* env = read_env_variable("BOTAN_MLOCK_POOL_SIZE"))
-      {
-      try
-         {
-         const size_t user_req = std::stoul(env, nullptr);
-         mlock_requested = std::min(user_req, mlock_requested);
-         }
-      catch(std::exception&) { /* ignore it */ }
-      }
+   const size_t mlock_requested = std::min<size_t>(user_req, BOTAN_MLOCK_ALLOCATOR_MAX_LOCKED_KB);
 
    if(mlock_requested > 0)
       {
@@ -302,22 +363,15 @@ size_t OS::get_memory_locking_limit()
 
    // According to Microsoft MSDN:
    // The maximum number of pages that a process can lock is equal to the number of pages in its minimum working set minus a small overhead
-   // In the book "Windows Internals Part 2": the maximum lockable pages are minimum working set size - 8 pages 
+   // In the book "Windows Internals Part 2": the maximum lockable pages are minimum working set size - 8 pages
    // But the information in the book seems to be inaccurate/outdated
    // I've tested this on Windows 8.1 x64, Windows 10 x64 and Windows 7 x86
    // On all three OS the value is 11 instead of 8
-   size_t overhead = OS::system_page_size() * 11ULL;
+   const size_t overhead = OS::system_page_size() * 11;
    if(working_min > overhead)
       {
-      size_t lockable_bytes = working_min - overhead;
-      if(lockable_bytes < (BOTAN_MLOCK_ALLOCATOR_MAX_LOCKED_KB * 1024ULL))
-         {
-         return lockable_bytes;
-         }
-      else
-         {
-         return BOTAN_MLOCK_ALLOCATOR_MAX_LOCKED_KB * 1024ULL;
-         }
+      const size_t lockable_bytes = working_min - overhead;
+      return std::min<size_t>(lockable_bytes, BOTAN_MLOCK_ALLOCATOR_MAX_LOCKED_KB * 1024);
       }
 #endif
 
@@ -333,73 +387,137 @@ const char* OS::read_env_variable(const std::string& name)
    return std::getenv(name.c_str());
    }
 
-void* OS::allocate_locked_pages(size_t length)
+size_t OS::read_env_variable_sz(const std::string& name, size_t def)
    {
-#if defined(BOTAN_TARGET_OS_HAS_POSIX1) && defined(BOTAN_TARGET_OS_HAS_POSIX_MLOCK)
+   if(const char* env = read_env_variable(name))
+      {
+      try
+         {
+         const size_t val = std::stoul(env, nullptr);
+         return val;
+         }
+      catch(std::exception&) { /* ignore it */ }
+      }
+
+   return def;
+   }
+
+std::vector<void*> OS::allocate_locked_pages(size_t count)
+   {
+   std::vector<void*> result;
+   result.reserve(count);
 
    const size_t page_size = OS::system_page_size();
 
-   if(length % page_size != 0)
-      return nullptr;
+   for(size_t i = 0; i != count; ++i)
+      {
+      void* ptr = nullptr;
 
-   void* ptr = nullptr;
-   int rc = ::posix_memalign(&ptr, page_size, length);
+#if defined(BOTAN_TARGET_OS_HAS_POSIX1) && defined(BOTAN_TARGET_OS_HAS_POSIX_MLOCK)
 
-   if(rc != 0 || ptr == nullptr)
-      return nullptr;
+#if !defined(MAP_NOCORE)
+#if defined(MAP_CONCEAL)
+   #define MAP_NOCORE MAP_CONCEAL
+#else
+   #define MAP_NOCORE 0
+#endif
+#endif
+#if !defined(PROT_MAX)
+   #define PROT_MAX(p) 0
+#endif
+      int pflags = PROT_READ | PROT_WRITE;
+
+      ptr = ::mmap(nullptr, 2*page_size,
+                   pflags | PROT_MAX(pflags),
+                   MAP_ANONYMOUS | MAP_PRIVATE | MAP_NOCORE,
+                   /*fd=*/-1, /*offset=*/0);
+
+      if(ptr == MAP_FAILED)
+         {
+         continue;
+         }
+
+      // failed to lock
+      if(::mlock(ptr, page_size) != 0)
+         {
+         ::munmap(ptr, 2*page_size);
+         continue;
+         }
 
 #if defined(MADV_DONTDUMP)
-   ::madvise(ptr, length, MADV_DONTDUMP);
+      // we ignore errors here, as DONTDUMP is just a bonus
+      ::madvise(ptr, page_size, MADV_DONTDUMP);
 #endif
 
-   if(::mlock(ptr, length) != 0)
-      {
-      std::free(ptr);
-      return nullptr; // failed to lock
-      }
-
-   ::memset(ptr, 0, length);
-
-   return ptr;
 #elif defined(BOTAN_TARGET_OS_HAS_VIRTUAL_LOCK)
-   LPVOID ptr = ::VirtualAlloc(nullptr, length, MEM_RESERVE | MEM_COMMIT, PAGE_READWRITE);
-   if(!ptr)
-      {
-      return nullptr;
+      ptr = ::VirtualAlloc(nullptr, 2*page_size, MEM_RESERVE | MEM_COMMIT, PAGE_READWRITE);
+
+      if(ptr == nullptr)
+         continue;
+
+      if(::VirtualLock(ptr, page_size) == 0)
+         {
+         ::VirtualFree(ptr, 0, MEM_RELEASE);
+         continue;
+         }
+#endif
+
+      std::memset(ptr, 0, 2*page_size); // zero both data and guard pages
+
+      // Make guard page following the data page
+      page_prohibit_access(static_cast<uint8_t*>(ptr) + page_size);
+
+      result.push_back(ptr);
       }
 
-   if(::VirtualLock(ptr, length) == 0)
-      {
-      ::VirtualFree(ptr, 0, MEM_RELEASE);
-      return nullptr; // failed to lock
-      }
+   return result;
+   }
 
-   return ptr;
-#else
-   BOTAN_UNUSED(length);
-   return nullptr; /* not implemented */
+void OS::page_allow_access(void* page)
+   {
+   const size_t page_size = OS::system_page_size();
+#if defined(BOTAN_TARGET_OS_HAS_POSIX1)
+   ::mprotect(page, page_size, PROT_READ | PROT_WRITE);
+#elif defined(BOTAN_TARGET_OS_HAS_VIRTUAL_LOCK)
+   DWORD old_perms = 0;
+   ::VirtualProtect(page, page_size, PAGE_READWRITE, &old_perms);
+   BOTAN_UNUSED(old_perms);
 #endif
    }
 
-void OS::free_locked_pages(void* ptr, size_t length)
+void OS::page_prohibit_access(void* page)
    {
-   if(ptr == nullptr || length == 0)
-      return;
+   const size_t page_size = OS::system_page_size();
+#if defined(BOTAN_TARGET_OS_HAS_POSIX1)
+   ::mprotect(page, page_size, PROT_NONE);
+#elif defined(BOTAN_TARGET_OS_HAS_VIRTUAL_LOCK)
+   DWORD old_perms = 0;
+   ::VirtualProtect(page, page_size, PAGE_NOACCESS, &old_perms);
+   BOTAN_UNUSED(old_perms);
+#endif
+   }
+
+void OS::free_locked_pages(const std::vector<void*>& pages)
+   {
+   const size_t page_size = OS::system_page_size();
+
+   for(size_t i = 0; i != pages.size(); ++i)
+      {
+      void* ptr = pages[i];
+
+      secure_scrub_memory(ptr, page_size);
+
+      // ptr points to the data page, guard page follows
+      page_allow_access(static_cast<uint8_t*>(ptr) + page_size);
 
 #if defined(BOTAN_TARGET_OS_HAS_POSIX1) && defined(BOTAN_TARGET_OS_HAS_POSIX_MLOCK)
-   secure_scrub_memory(ptr, length);
-   ::munlock(ptr, length);
-   std::free(ptr);
-
+      ::munlock(ptr, page_size);
+      ::munmap(ptr, 2*page_size);
 #elif defined(BOTAN_TARGET_OS_HAS_VIRTUAL_LOCK)
-   secure_scrub_memory(ptr, length);
-   ::VirtualUnlock(ptr, length);
-   ::VirtualFree(ptr, 0, MEM_RELEASE);
-
-#else
-   // Invalid argument because no way this pointer was allocated by us
-   throw Invalid_Argument("Invalid ptr to free_locked_pages");
+      ::VirtualUnlock(ptr, page_size);
+      ::VirtualFree(ptr, 0, MEM_RELEASE);
 #endif
+      }
    }
 
 #if defined(BOTAN_TARGET_OS_HAS_POSIX1) && !defined(BOTAN_TARGET_OS_IS_EMSCRIPTEN)
