@@ -176,17 +176,13 @@ std::unique_ptr<X509_Certificate_Data> parse_x509_cert_body(const X509_Object& o
                throw Decoding_Error("Algorithm identifier mismatch");
                }
             }
-         if(public_key_info[1] == "OAEP")
-            {
-            throw Decoding_Error("Decoding subject public keys of type RSAES-OAEP is currently not supported");
-            }
          }
       else
          {
          // oid = rsaEncryption -> parameters field MUST contain NULL
          if(public_key_alg_id != AlgorithmIdentifier(public_key_alg_id.get_oid(), AlgorithmIdentifier::USE_NULL_PARAM))
             {
-            throw Decoding_Error("Parameters field MUST contain NULL");
+            throw Decoding_Error("RSA algorithm parameters field MUST contain NULL");
             }
          }
       }
@@ -201,10 +197,13 @@ std::unique_ptr<X509_Certificate_Data> parse_x509_cert_body(const X509_Object& o
 
    if(v3_exts_data.is_a(3, ASN1_Tag(CONSTRUCTED | CONTEXT_SPECIFIC)))
       {
+      // Path validation will reject a v1/v2 cert with v3 extensions
       BER_Decoder(v3_exts_data).decode(data->m_v3_extensions).verify_end();
       }
    else if(v3_exts_data.is_set())
+      {
       throw BER_Bad_Tag("Unknown tag in X.509 cert", v3_exts_data.tagging());
+      }
 
    // Now cache some fields from the extensions
    if(auto ext = data->m_v3_extensions.get_extension_object_as<Cert_Extension::Key_Usage>())
@@ -284,6 +283,13 @@ std::unique_ptr<X509_Certificate_Data> parse_x509_cert_body(const X509_Object& o
          }
       else
          {
+         /*
+         If a parse error or unknown algorithm is encountered, default
+         to assuming it is self signed. We have no way of being certain but
+         that is usually the default case (self-issued is rare in practice).
+         */
+         data->m_self_signed = true;
+
          try
             {
             std::unique_ptr<Public_Key> pub_key(X509::load_key(data->m_subject_public_key_bits_seq));
@@ -295,6 +301,10 @@ std::unique_ptr<X509_Certificate_Data> parse_x509_cert_body(const X509_Object& o
                {
                data->m_self_signed = true;
                }
+            else
+               {
+               data->m_self_signed = false;
+               }
             }
          catch(...)
             {
@@ -442,11 +452,17 @@ const std::vector<uint8_t>& X509_Certificate::raw_subject_dn() const
 
 bool X509_Certificate::is_CA_cert() const
    {
+   if(data().m_version < 3 && data().m_self_signed)
+      return true;
+
    return data().m_is_ca_certificate;
    }
 
 uint32_t X509_Certificate::path_limit() const
    {
+   if(data().m_version < 3 && data().m_self_signed)
+      return 32; // in theory infinite, but this is more than enough
+
    return static_cast<uint32_t>(data().m_path_len_constraint);
    }
 
@@ -793,20 +809,27 @@ std::string X509_Certificate::to_string() const
          out << "   Decipher Only\n";
       }
 
-   const std::vector<OID> policies = this->certificate_policy_oids();
+   const std::vector<OID>& policies = this->certificate_policy_oids();
    if(!policies.empty())
       {
       out << "Policies: " << "\n";
       for(auto oid : policies)
-         out << "   " << oid.as_string() << "\n";
+         out << "   " << oid.to_string() << "\n";
       }
 
-   std::vector<OID> ex_constraints = this->extended_key_usage();
+   const std::vector<OID>& ex_constraints = this->extended_key_usage();
    if(!ex_constraints.empty())
       {
       out << "Extended Constraints:\n";
-      for(size_t i = 0; i != ex_constraints.size(); i++)
-         out << "   " << OIDS::oid2str(ex_constraints[i]) << "\n";
+      for(auto&& oid : ex_constraints)
+         {
+         const std::string oid_str = OIDS::oid2str(oid);
+
+         if(oid_str.empty())
+            out << "   " << oid.to_string() << "\n";
+         else
+            out << "   " << oid_str << "\n";
+         }
       }
 
    const NameConstraints& name_constraints = this->name_constraints();
@@ -839,7 +862,7 @@ std::string X509_Certificate::to_string() const
    if(!ocsp_responder().empty())
       out << "OCSP responder " << ocsp_responder() << "\n";
 
-   std::vector<std::string> ca_issuers = this->ca_issuers();
+   const std::vector<std::string> ca_issuers = this->ca_issuers();
    if(!ca_issuers.empty())
       {
       out << "CA Issuers:\n";
@@ -870,7 +893,7 @@ std::string X509_Certificate::to_string() const
    catch(Decoding_Error&)
       {
       const AlgorithmIdentifier& alg_id = this->subject_public_key_algo();
-      out << "Failed to decode key with oid " << alg_id.get_oid().as_string() << "\n";
+      out << "Failed to decode key with oid " << alg_id.get_oid().to_string() << "\n";
       }
 
    return out.str();
