@@ -22,7 +22,7 @@ def get_concurrency():
 
     try:
         import multiprocessing
-        return max(def_concurrency, multiprocessing.cpu_count())
+        return multiprocessing.cpu_count()
     except ImportError:
         return def_concurrency
 
@@ -48,12 +48,15 @@ def determine_flags(target, target_os, target_cpu, target_cc, cc_bin, ccache, ro
     test_prefix = []
     test_cmd = [os.path.join(root_dir, 'botan-test')]
 
+    if target in ['shared', 'static', 'sanitizer', 'fuzzers', 'gcc4.8', 'cross-i386', 'bsi', 'nist']:
+        test_cmd += ['--test-threads=%d' % (get_concurrency())]
+
     fast_tests = ['block', 'aead', 'hash', 'stream', 'mac', 'modes', 'kdf',
                   'hmac_drbg', 'hmac_drbg_unit',
                   'tls', 'ffi',
                   'rsa_sign', 'rsa_verify', 'dh_kat',
                   'ecc_randomized', 'ecdh_kat', 'ecdsa_sign', 'curve25519_scalar',
-                  'simd_32', 'os_utils', 'util', 'util_dates']
+                  'cpuid', 'simd_32', 'os_utils', 'util', 'util_dates']
 
     install_prefix = os.path.join(tempfile.gettempdir(), 'botan-install')
     flags = ['--prefix=%s' % (install_prefix),
@@ -72,10 +75,8 @@ def determine_flags(target, target_os, target_cpu, target_cc, cc_bin, ccache, ro
     if target in ['mini-static', 'mini-shared']:
         flags += ['--minimized-build', '--enable-modules=system_rng,sha2_32,sha2_64,aes']
 
-    if target == 'shared' and target_os != 'osx':
-        # Enabling amalgamation build for shared is somewhat arbitrary, but we want to test it
-        # somewhere. In addition the majority of the Windows builds are shared, and MSVC is
-        # much faster compiling via the amalgamation than individual files.
+    if target == 'static':
+        # Arbitrarily test amalgamation with the static lib builds
         flags += ['--amalgamation']
 
         if target_cc == 'msvc':
@@ -93,7 +94,7 @@ def determine_flags(target, target_os, target_cpu, target_cc, cc_bin, ccache, ro
         test_cmd = None
 
     if target == 'coverage':
-        flags += ['--with-coverage-info', '--test-mode']
+        flags += ['--with-coverage-info', '--test-mode', '--build-bogo-shim']
     if target == 'valgrind':
         # valgrind in 16.04 has a bug with rdrand handling
         flags += ['--with-valgrind', '--disable-rdrand']
@@ -107,13 +108,7 @@ def determine_flags(target, target_os, target_cpu, target_cc, cc_bin, ccache, ro
     if target in ['fuzzers', 'coverage']:
         flags += ['--build-fuzzers=test']
     if target in ['fuzzers', 'sanitizer']:
-
-        # On VC iterator debugging comes from generic debug mode
-        if target_cc == 'msvc':
-            flags += ['--with-debug-info']
-        else:
-            flags += ['--with-sanitizers']
-        flags += ['--with-debug-asserts']
+        flags += ['--with-sanitizers', '--with-debug-asserts']
 
     if target in ['valgrind', 'sanitizer', 'fuzzers']:
         flags += ['--disable-modules=locking_allocator']
@@ -154,7 +149,7 @@ def determine_flags(target, target_os, target_cpu, target_cc, cc_bin, ccache, ro
             cc_bin = 'x86_64-w64-mingw32-g++'
             flags += ['--cpu=x86_64', '--cc-abi-flags=-static',
                       '--ar-command=x86_64-w64-mingw32-ar', '--without-os-feature=threads']
-            test_cmd = [os.path.join(root_dir, 'botan-test.exe')]
+            test_cmd = [os.path.join(root_dir, 'botan-test.exe')] + test_cmd[1:]
             # No runtime prefix required for Wine
         else:
             # Build everything but restrict what is run
@@ -194,11 +189,18 @@ def determine_flags(target, target_os, target_cpu, target_cc, cc_bin, ccache, ro
         if target_os in ['osx', 'ios']:
             flags += ['--with-commoncrypto']
 
-        if target_os == 'osx':
-            # Test Boost on OS X
+        if target_os == 'osx' or target == 'coverage':
             flags += ['--with-boost']
-            # Travis has 10.12 as default image
-            flags += ['--with-os-features=getentropy']
+
+        if target_os == 'windows' and target in ['shared', 'static']:
+            # ./configure.py needs extra hand-holding for boost on windows
+            boost_root = os.environ['BOOST_ROOT']
+            boost_libs = os.environ['BOOST_LIBRARYDIR']
+            boost_system = os.environ['BOOST_SYSTEM_LIBRARY']
+            flags += ['--with-boost',
+                      '--with-external-includedir', boost_root,
+                      '--with-external-libdir', boost_libs,
+                      '--boost-library-name', boost_system]
 
         if target_os == 'linux':
             flags += ['--with-lzma']
@@ -242,14 +244,18 @@ def run_cmd(cmd, root_dir):
 
     cmd = [os.path.expandvars(elem) for elem in cmd]
     sub_env = os.environ.copy()
-    sub_env['LD_LIBRARY_PATH'] = root_dir
-    sub_env['PYTHONPATH'] = os.path.join(root_dir, 'src/python')
+    sub_env['LD_LIBRARY_PATH'] = os.path.abspath(root_dir)
+    sub_env['PYTHONPATH'] = os.path.abspath(os.path.join(root_dir, 'src/python'))
+    cwd = None
 
     redirect_stdout = None
     if len(cmd) > 3 and cmd[-2] == '>':
         redirect_stdout = open(cmd[-1], 'w')
         cmd = cmd[:-2]
-    proc = subprocess.Popen(cmd, close_fds=True, env=sub_env, stdout=redirect_stdout)
+    if len(cmd) > 1 and cmd[0].startswith('indir:'):
+        cwd = cmd[0][6:]
+        cmd = cmd[1:]
+    proc = subprocess.Popen(cmd, cwd=cwd, close_fds=True, env=sub_env, stdout=redirect_stdout)
     proc.communicate()
 
     time_taken = int(time.time() - start)
@@ -302,10 +308,10 @@ def parse_args(args):
     parser.add_option('--build-jobs', metavar='J', default=get_concurrency(),
                       help='Set number of jobs to run in parallel (default %default)')
 
-    parser.add_option('--compiler-cache', default=None,
-                      help='Set a compiler cache to use (ccache, clcache)')
+    parser.add_option('--compiler-cache', default=None, metavar='CC',
+                      help='Set a compiler cache to use (ccache, sccache, clcache)')
 
-    parser.add_option('--pkcs11-lib', default=None,
+    parser.add_option('--pkcs11-lib', default=None, metavar='LIB',
                       help='Set PKCS11 lib to use for testing')
 
     parser.add_option('--with-python3', dest='use_python3', action='store_true', default=None,
@@ -348,16 +354,16 @@ def main(args=None):
 
     target = args[1]
 
-    py_interp = 'python'
-
     use_python2 = have_prog('python2')
 
     if options.use_python3 is None:
         use_python3 = have_prog('python3')
     else:
         use_python3 = options.use_python3
-        if use_python3:
-            py_interp = 'python3'
+
+    py_interp = 'python'
+    if use_python3:
+        py_interp = 'python3'
 
     if options.cc_bin is None:
         if options.cc == 'gcc':
@@ -450,25 +456,41 @@ def main(args=None):
         if target == 'docs':
             cmds.append(make_cmd + ['docs'])
         else:
-            if options.compiler_cache == 'ccache':
-                cmds.append(['ccache', '--show-stats'])
-            elif options.compiler_cache == 'clcache':
-                cmds.append(['clcache', '-s'])
 
-            make_targets = ['libs', 'cli', 'tests']
+            ccache_show_stats = {
+                'ccache': '--show-stats',
+                'sccache': '--show-stats',
+                'clcache': '-s'
+            }
+
+            if options.compiler_cache in ccache_show_stats:
+                cmds.append([options.compiler_cache, ccache_show_stats[options.compiler_cache]])
+
+            make_targets = ['libs', 'tests', 'cli']
+
             if target in ['coverage', 'fuzzers']:
-                make_targets += ['fuzzers', 'fuzzer_corpus_zip']
+                make_targets += ['tests', 'cli', 'fuzzers', 'fuzzer_corpus_zip']
+
+            if target in ['coverage']:
+                make_targets += ['bogo_shim']
 
             cmds.append(make_prefix + make_cmd + make_targets)
 
-            if options.compiler_cache == 'ccache':
-                cmds.append(['ccache', '--show-stats'])
-            elif options.compiler_cache == 'clcache':
-                cmds.append(['clcache', '-s'])
+            if options.compiler_cache in ccache_show_stats:
+                cmds.append([options.compiler_cache, ccache_show_stats[options.compiler_cache]])
 
         if run_test_command is not None:
             cmds.append(run_test_command)
 
+        if target == 'coverage':
+            runner_dir = os.path.abspath(os.path.join(root_dir, 'boringssl', 'ssl', 'test', 'runner'))
+
+            cmds.append(['indir:%s' % (runner_dir),
+                         'go', 'test', '-pipe',
+                         '-num-workers', str(4*get_concurrency()),
+                         '-shim-path', os.path.abspath(os.path.join(root_dir, 'botan_bogo_shim')),
+                         '-shim-config', os.path.abspath(os.path.join(root_dir, 'src', 'bogo_shim', 'config.json'))])
+
         if target in ['coverage', 'fuzzers']:
             cmds.append([py_interp, os.path.join(root_dir, 'src/scripts/test_fuzzers.py'),
                          os.path.join(root_dir, 'fuzzer_corpus'),
@@ -500,7 +522,7 @@ def main(args=None):
             cmds.append(['llvm-cov', 'show', './botan-test',
                          '-instr-profile=botan.profdata',
                          '>', 'build/cov_report.txt'])
-            sonar_config = os.path.join(root_dir, os.path.join(root_dir, 'src/configs/sonar-project.properties'))
+            sonar_config = os.path.join(root_dir, 'src/configs/sonar-project.properties')
             cmds.append(['sonar-scanner',
                          '-Dproject.settings=%s' % (sonar_config),
                          '-Dsonar.login=$SONAR_TOKEN'])
