@@ -60,12 +60,12 @@ class Test_Options
                    const std::string& provider,
                    const std::string& drbg_seed,
                    size_t test_runs,
+                   size_t test_threads,
                    bool verbose,
                    bool log_success,
                    bool run_online_tests,
                    bool run_long_tests,
-                   bool abort_on_first_fail,
-                   bool undefined_behavior_allowed) :
+                   bool abort_on_first_fail) :
          m_requested_tests(requested_tests),
          m_skip_tests(skip_tests.begin(), skip_tests.end()),
          m_data_dir(data_dir),
@@ -73,12 +73,12 @@ class Test_Options
          m_provider(provider),
          m_drbg_seed(drbg_seed),
          m_test_runs(test_runs),
+         m_test_threads(test_threads),
          m_verbose(verbose),
          m_log_success(log_success),
          m_run_online_tests(run_online_tests),
          m_run_long_tests(run_long_tests),
-         m_abort_on_first_fail(abort_on_first_fail),
-         m_undefined_behavior_allowed(undefined_behavior_allowed)
+         m_abort_on_first_fail(abort_on_first_fail)
          {
          }
 
@@ -98,6 +98,8 @@ class Test_Options
 
       size_t test_runs() const { return m_test_runs; }
 
+      size_t test_threads() const { return m_test_threads; }
+
       bool log_success() const { return m_log_success; }
 
       bool run_online_tests() const { return m_run_online_tests; }
@@ -108,16 +110,6 @@ class Test_Options
 
       bool verbose() const { return m_verbose; }
 
-      bool undefined_behavior_allowed() const
-         {
-#if defined(BOTAN_HAS_SANITIZER_UNDEFINED)
-         return m_undefined_behavior_allowed;
-#else
-         BOTAN_UNUSED(m_undefined_behavior_allowed);
-         return true;
-#endif
-         }
-
    private:
       std::vector<std::string> m_requested_tests;
       std::set<std::string> m_skip_tests;
@@ -126,12 +118,12 @@ class Test_Options
       std::string m_provider;
       std::string m_drbg_seed;
       size_t m_test_runs;
+      size_t m_test_threads;
       bool m_verbose;
       bool m_log_success;
       bool m_run_online_tests;
       bool m_run_long_tests;
       bool m_abort_on_first_fail;
-      bool m_undefined_behavior_allowed;
    };
 
 /*
@@ -261,7 +253,7 @@ class Test
 
                if(produced == expected)
                   {
-                  out << " produced expected result " << produced;
+                  out << " produced expected result";
                   return test_success(out.str());
                   }
                else
@@ -436,22 +428,19 @@ class Test
             std::vector<std::string> m_log;
          };
 
-      class Registration final
-         {
-         public:
-            Registration(const std::string& name, Test* test);
-         };
-
       virtual ~Test() = default;
       virtual std::vector<Test::Result> run() = 0;
 
       virtual std::vector<std::string> possible_providers(const std::string&);
 
-      static std::map<std::string, std::unique_ptr<Test>>& global_registry();
+      static void register_test(const std::string& name,
+                                std::function<Test* ()> maker_fn);
+
+      static std::map<std::string, std::function<Test* ()>>& global_registry();
 
       static std::set<std::string> registered_tests();
 
-      static Test* get_test(const std::string& test_name);
+      static std::unique_ptr<Test> get_test(const std::string& test_name);
 
       static std::string data_file(const std::string& what);
 
@@ -514,8 +503,48 @@ class Test
 /*
 * Register the test with the runner
 */
-#define BOTAN_REGISTER_TEST(type, Test_Class) \
-   Test::Registration reg_ ## Test_Class ## _tests(type, new Test_Class)
+template<typename Test_Class>
+class TestClassRegistration
+   {
+   public:
+      TestClassRegistration(const std::string& name)
+         {
+         auto test_maker = []() -> Test* { return new Test_Class; };
+         Test::register_test(name, test_maker);
+         }
+   };
+
+#define BOTAN_REGISTER_TEST(name, Test_Class)                 \
+   TestClassRegistration<Test_Class> reg_ ## Test_Class ## _tests(name)
+
+typedef Test::Result (*test_fn)();
+
+class FnTest : public Test
+   {
+   public:
+      FnTest(test_fn fn) : m_fn(fn) {}
+
+      std::vector<Test::Result> run() override
+         {
+         return {m_fn()};
+         }
+
+   private:
+      test_fn m_fn;
+   };
+
+class TestFnRegistration
+   {
+   public:
+      TestFnRegistration(const std::string& name, test_fn fn)
+         {
+         auto test_maker = [=]() -> Test* { return new FnTest(fn); };
+         Test::register_test(name, test_maker);
+         }
+   };
+
+#define BOTAN_REGISTER_TEST_FN(name, fn_name)         \
+   TestFnRegistration reg_ ## fn_name(name, fn_name)
 
 class VarMap
    {
